package pwc.taxtech.atms.controller;

import com.github.pagehelper.PageInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dto.vatdto.ImportInputInvoiceDto;
import pwc.taxtech.atms.dto.vatdto.ImportInputInvoiceItemDto;
import pwc.taxtech.atms.dto.vatdto.InputVATInvoiceBaseDto;
import pwc.taxtech.atms.service.impl.IdentityServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelSheet;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.dpo.InputInvoicePreviewQueryParam;
import pwc.taxtech.atms.vat.entity.InputInvoice;
import pwc.taxtech.atms.vat.entity.InputInvoiceDetail;
import pwc.taxtech.atms.vat.entity.InputVatInvoiceItem;
import pwc.taxtech.atms.vat.service.impl.InputInvoiceDataImportServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;

import static javax.servlet.http.HttpServletResponse.SC_NO_CONTENT;
import static javax.servlet.http.HttpServletResponse.SC_OK;

@RestController
@RequestMapping(value = "api/v1/inputInvoiceImport")
public class InputInvoiceImportController {
    @Autowired
    InputInvoiceDataImportServiceImpl inputInvoiceDataImportService;
    @Autowired
    private IdentityServiceImpl identityService;
    private Logger logger = LoggerFactory.getLogger(InputInvoiceImportController.class);

    @RequestMapping(value = "inputInvoicePreviewList", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    PageInfo<InputInvoice> getInputInvoiceTreeViewData(@RequestBody InputInvoicePreviewQueryParam paras,
                                                       @RequestHeader("from") String projectId) {
        return inputInvoiceDataImportService.getInputInvoiceTreeViewData(paras, projectId);
    }

    @RequestMapping(value = "exportQueryData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadInvoiceQueryData(@RequestBody InputInvoicePreviewQueryParam paras, HttpServletResponse response,
                                         @RequestHeader("from") String projectId) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + UUID.randomUUID() + ".xlsx");
        response.setHeader("x-file-name", UUID.randomUUID() + ".xlsx");

        OutputStream os = null;
        try {
            os = response.getOutputStream();
            int count = getDownloadFilePath(paras, os, projectId);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @RequestMapping(value = "getInputInvoiceList/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getInputInvoiceList(@PathVariable Integer period) {
        return ResponseEntity.ok().body(inputInvoiceDataImportService.getInputInvoiceList(period));
    }

    @RequestMapping(value = "getInputInvoiceItemList/{fpid}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getInputInvoiceItemList(@PathVariable String fpid) {
        return ResponseEntity.ok().body(inputInvoiceDataImportService.getInputInvoiceItemList(fpid));
    }

    private int getDownloadFilePath(InputInvoicePreviewQueryParam paras, OutputStream outputStream, String projectId) {
        List<InputInvoice> list = inputInvoiceDataImportService.getInputInvoiceTreeViewData(paras, projectId).getList();
        if (list.size() == 0) {
            return 0;
        }
        Map<String, String> header = new HashMap<>();
        header.put("PeriodId", "期间");
        header.put("InvoiceDate", "开票日期");
        header.put("InvoiceCode", "发票代码");
        header.put("InvoiceNumber", "发票号码");
        header.put("SellerTaxNumber", "供货方税号");
        header.put("InvoiceTypeName", "发票类型");
        header.put("Amount", "金额");
        header.put("TaxAmount", "税额");
        header.put("CertificationDate", "认证日期");

        ExcelSheet excelSheetA = new ExcelSheet<>();
        excelSheetA.setHeaders(header);
        excelSheetA.setDataset(list);
        excelSheetA.setSheetName("sheetA");


        Map<String, String> header2 = new HashMap<>();
        header2.put("PeriodId", "期间");
        header2.put("InvoiceCode", "发票代码");
        header2.put("InvoiceNumber", "发票号码");
        header2.put("Amount", "金额");
        header2.put("TaxAmount", "税额");
        header2.put("TaxRate", "税率");
        header2.put("ProductionName", "货物名称");

        ExcelSheet excelSheetB = new ExcelSheet<>();
        excelSheetB.setHeaders(header2);
        excelSheetB.setSheetName("sheetB");

        List<InputInvoiceDetail> inputInvoiceItemList = new ArrayList<>();
        for (InputInvoice item : list) {
            inputInvoiceItemList.addAll(inputInvoiceDataImportService.getInputInvoiceItemList(item.getID()));
        }
        excelSheetB.setDataset(inputInvoiceItemList);
        List<ExcelSheet<InputVATInvoiceBaseDto>> sheets = new ArrayList<>();
        sheets.add(excelSheetA);
        sheets.add(excelSheetB);

        ExcelUtil.exportExcel(sheets, outputStream);
        return list.size();
    }

    @RequestMapping(value = "importInputInvoice", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity ImportInputInvoiceData(@RequestBody ImportInputInvoiceDto importParam) {
        return ResponseEntity.ok(inputInvoiceDataImportService.ImportInputInvoiceData(importParam.InputInvoiceList,
                importParam.ImportType, identityService.getIdentityUser().getId()));
    }

    @RequestMapping(value = "importInputInvoiceItem", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity ImportInputInvoiceItemData(@RequestBody ImportInputInvoiceItemDto importParam) {
        return ResponseEntity.ok(inputInvoiceDataImportService.ImportInputInvoiceItemData(importParam.InputInvoiceItemList,
                importParam.ImportType, identityService.getIdentityUser().getId()));
    }

    @RequestMapping(value = "inputInvoiceItemPreviewList/{InputInvoiceId}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity GetInputInvoiceItemTreeViewData(@PathVariable String InputInvoiceId) {
        return ResponseEntity.ok(inputInvoiceDataImportService.getInputInvoiceItemList(InputInvoiceId));
    }
}
