package pwc.taxtech.atms.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.dpo.ProjectDisplayDto;
import pwc.taxtech.atms.dto.ProjectClientDto;
import pwc.taxtech.atms.dto.ServiceTypeDto;
import pwc.taxtech.atms.dto.taxadmin.AddProjectResult;
import pwc.taxtech.atms.service.impl.ProjectServiceImpl;

import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1/project")
public class ProjectController {
    private static Logger logger = LoggerFactory.getLogger(ProjectController.class);

    @Autowired
    private ProjectServiceImpl projectService;

    @ApiOperation(value = "listService", notes = "Get service list")
    @RequestMapping(value = "listService", method = RequestMethod.GET)
    public @ResponseBody
    List<ServiceTypeDto> getServiceList() {
        logger.info("/api/v1/project/listService");
        return projectService.getServiceList();
    }

    @ApiOperation(value = "getProjectClientList", notes = "Get project Client list")
    @RequestMapping(value = "getProjectClientList", method = RequestMethod.GET)
    public @ResponseBody
    List<ProjectClientDto> getProjectClientList() {
        logger.info("/api/v1/project/getProjectClientList");
        return projectService.getProjectClientList();
    }

    @ApiOperation(value = "getProject", notes = "Get Project")
    @RequestMapping(value = "getProject", method = RequestMethod.GET)
    public @ResponseBody
    List<ProjectDisplayDto> GetProjectById(String projectId) {
        return projectService.getProjectById(projectId);
    }

    @ApiOperation(value = "getAllProjectList", notes = "Get All  List")
    @RequestMapping(value = "getAllProjectList", method = RequestMethod.GET)
    public @ResponseBody
    List<ProjectDisplayDto> getAllProjectList(String orgId, String serviceId, Integer projectYear) {
        logger.info("/api/v1/project/getAllProjectList with orgId {} serviceId {}", orgId, serviceId);
        return projectService.getAllProjectList(orgId, serviceId == null ? "" : serviceId, projectYear);
    }

    @ApiOperation(value = "add", notes = "add a project")
    @RequestMapping(value = "add", method = RequestMethod.POST)
    @ApiImplicitParam(name = "projectDto", value = "projectDto", required = true, dataType = "ProjectDisplayDto")
    public @ResponseBody
    AddProjectResult AddProject(@RequestBody ProjectDisplayDto projectDto, String userName) {
        if (userName == null || userName.isEmpty()) userName = "test";//TODO: should use session username future(neo)
        return projectService.addProject(projectDto, userName);
    }

    @ApiOperation(value = "getProjectAllStatus", notes = "get project all status")
    @RequestMapping(value = "getProjectAllStatus/{id}", method = RequestMethod.GET)
    public @ResponseBody
    Map<Integer, Integer> getProjectAllStatus(@PathVariable String id) {
        return projectService.getProjectAllStatus(id);
    }

}
