package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dao.FormulaAdminMapper;
import pwc.taxtech.atms.dao.ProjectServiceTypeMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dpo.GroupId;
import pwc.taxtech.atms.entity.ProjectServiceType;
import pwc.taxtech.atms.entity.ProjectServiceTypeExample;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.CellDataMapper;
import pwc.taxtech.atms.vat.dao.PeriodCellDataMapper;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.entity.PeriodCellData;
import pwc.taxtech.atms.vat.entity.PeriodCellDataExample;
import pwc.taxtech.atms.vat.entity.PeriodReport;
import pwc.taxtech.atms.vat.entity.PeriodReportExample;

import java.util.List;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Component
public class FormulaAgent {
    @Autowired
    private FormulaAdminMapper adminMp;
    @Autowired
    private PeriodReportMapper periodReportMapper;
    @Autowired
    private CellDataMapper cellDataMapper;
    @Autowired
    private PeriodCellDataMapper periodCellDataMapper;
    @Autowired
    private ProjectServiceTypeMapper projectServiceTypeMapper;

    public List<CellTemplatePerGroupDto> getCellTemplateGroupDto(Long templateGroupId, String projectId) {
        ProjectServiceTypeExample pst = new ProjectServiceTypeExample();
        pst.createCriteria().andTemplateGroupIdEqualTo(Long.valueOf(templateGroupId))
                .andServiceTypeIdEqualTo(EnumServiceType.VAT.getCode() + "");
        List<ProjectServiceType> pstReult = projectServiceTypeMapper.selectByExample(pst);

        List<CellTemplatePerGroupDto> cellTemplates = adminMp.getCellTemplatePerGroupDto(templateGroupId);
        if (pstReult == null || pstReult.isEmpty()) {
            List<GroupId> groupIds = adminMp.getTemplateGroupId(projectId);
            GroupId groupId;

            if (groupIds.size() > 0) {
                groupId = groupIds.stream().filter(m -> m.isDefault != 1).findFirst().get();
                if (groupId == null) groupId = groupIds.get(0);
                cellTemplates.addAll(adminMp.getCellTemplatePerGroupDto(groupId.groupId));
            }
        }
        return cellTemplates;
    }

    private PeriodReport getReportByTemplate(String templateId, Integer periodId, String projectId) {
        PeriodReportExample example = new PeriodReportExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(Long.valueOf(templateId))
                .andPeriodEqualTo(periodId);
        List<PeriodReport> reports = periodReportMapper.selectByExample(example);

        if (reports != null && !reports.isEmpty()) return reports.get(FIRST_OR_DEFAULT);
        return null;
    }

    private PeriodCellData getCellDataListByTemplate(String templateId, Long reportId) {
        PeriodCellDataExample dataExample = new PeriodCellDataExample();
        dataExample.createCriteria().andCellTemplateIdEqualTo(Long.valueOf(templateId)).andReportIdEqualTo(reportId);
        List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(dataExample);

        if (cellDataList != null && !cellDataList.isEmpty()) return cellDataList.get(FIRST_OR_DEFAULT);
        return null;
    }

    public String getPastProjectId(int year, String orgId) {
        return adminMp.getPastProjectId(year, orgId);
    }

    public PeriodCellData getCellData(String templateId, String cellId, int periodId, String projectId) {
        PeriodReport report = getReportByTemplate(templateId, periodId, projectId);
        MyAsserts.assertNotNull(report, Exceptions.BB_REPORT_NULL);

        PeriodCellData cellData = getCellDataListByTemplate(cellId, report.getId());
        MyAsserts.assertNotNull(cellData, Exceptions.BB_CELL_DATA_NULL);
        return cellData;
    }

}
