package pwc.taxtech.atms.vat.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.constant.enums.EnumValidationType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dto.FieldsMapper;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.InputVATInvoiceDto;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.invoice.InputInvoiceDetailMapper;
import pwc.taxtech.atms.invoice.InputInvoiceMapper;
import pwc.taxtech.atms.vat.dao.InputVatInvoiceItemMapper;
import pwc.taxtech.atms.vat.dao.InputVatInvoiceMapper;
import pwc.taxtech.atms.vat.dpo.InputInvoicePreviewQueryParam;
import pwc.taxtech.atms.vat.entity.*;

import java.io.StringReader;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;

import static pwc.taxtech.atms.constant.Constant.INPUT_RZJG_SUCCESS;
import static pwc.taxtech.atms.constant.Constant.INPUT_RZZT_OVER;

@Service
public class InputInvoiceDataImportServiceImpl {
    private static final String _tableName = "InputVATInvoice";
    private static final String _itemTableName = "InputVATInvoiceItem";
    private static final String _validationTableName = "ValidationInfo";

    @Autowired
    private InputVatInvoiceMapper inputVATInvoiceMapper;
    @Autowired
    private SqlSessionTemplate dynamicSqlSessionTemplate;
    @Autowired
    private InputVatInvoiceItemMapper inputVATInvoiceItemMapper;
    @Autowired
    private InputInvoiceMapper inputInvoiceMapper;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private InputInvoiceDetailMapper inputInvoiceDetailMapper;
    @Autowired
    private OrganizationMapper organizationMapper;

    public PageInfo<InputInvoice> getInputInvoiceTreeViewData(InputInvoicePreviewQueryParam paras, String projectId) {
        Project project = projectMapper.selectByPrimaryKey(projectId);
        Organization organization = organizationMapper.selectByPrimaryKey(project.getOrganizationId());
        InputInvoiceExample invoiceExample = new InputInvoiceExample();
        invoiceExample.createCriteria().andGFSHEqualTo(organization.getTaxPayerNumber()).andRZSJBetween(DateUtils.getPeriodBegin(
                project.getYear(), paras.getPeriodStart()), DateUtils.getPeriodEnd(project.getYear(), paras.getPeriodEnd()))
                .andRZJGEqualTo(INPUT_RZJG_SUCCESS).andRZZTEqualTo(INPUT_RZZT_OVER);

        PageHelper.startPage(paras.getPageInfo().getPageIndex(), paras.getPageInfo().getPageSize());
        List<InputInvoice> invoices = inputInvoiceMapper.selectByExample(invoiceExample);
        DecimalFormat df = new DecimalFormat("#,###.00");
        invoices.stream().forEach(x -> {
            x.setHJJE(df.format(new BigDecimal(x.getHJJE())));
            x.setHJSE(df.format(new BigDecimal(x.getHJSE())));
                }
        );
        PageInfo<InputInvoice> pageInfo = new PageInfo<>(invoices);
        return pageInfo;
    }


    public List<InputVATInvoiceDto> getInputInvoiceList(Integer period) {

        InputVatInvoiceExample inputVATInvoiceExample = new InputVatInvoiceExample();
        inputVATInvoiceExample.createCriteria().andPeriodIdEqualTo(period.intValue());
        List<InputVatInvoice> inputVatEntity = inputVATInvoiceMapper.selectByExample(inputVATInvoiceExample);
        inputVatEntity.sort(Comparator.comparing(InputVatInvoice::getInvoiceNumber));
        inputVatEntity.sort(Comparator.comparing(InputVatInvoice::getCreateTime));
        List<InputVATInvoiceDto> result = new ArrayList<>(inputVatEntity.size());

        inputVatEntity.forEach(m -> {
            InputVATInvoiceDto dto = new InputVATInvoiceDto();
            try {
                FieldsMapper.map(m, dto);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();//TODO:should be fixed with exception filter (neo)
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            result.add(dto);
        });

        return result;
    }

    public List<InputInvoiceDetail> getInputInvoiceItemList(String fpid) {
        InputInvoice inputInvoice = inputInvoiceMapper.selectByPrimaryKey(fpid);
        InputInvoiceDetailExample example = new InputInvoiceDetailExample();
        example.createCriteria().andFPHMEqualTo(inputInvoice.getFPHM());
        return inputInvoiceDetailMapper.selectByExample(example);
    }

    public OperationResultDto ImportInputInvoiceItemData(List<InputVatInvoiceItem> inputInvoiceItemDtoList, int importType, String userId) {
        if (inputInvoiceItemDtoList.size() > 0) {
            int period = inputInvoiceItemDtoList.get(0).getPeriodId();
            inputInvoiceItemDtoList.forEach(item -> {
                //导入之前需要将汇总表的主键更新到明细表中,如果找不到汇总信息，则设置为空
                InputVatInvoiceExample example = new InputVatInvoiceExample();
                example.createCriteria().andInvoiceCodeEqualTo(item.getInvoiceCode()).andInvoiceNumberEqualTo(item.getInvoiceNumber());
                InputVatInvoice invoiceTotal = inputVATInvoiceMapper.selectByExample(example).stream().findFirst().get();

                item.setInputVATInvoiceId(invoiceTotal == null ? "" : invoiceTotal.getId());
                item.setId(UUID.randomUUID().toString());
                item.setCreatorId(userId);
                item.setCreateTime(new Date());
            });

            //如果是覆盖导入，则先删除该期间的数据，再进行数据导入
            if (importType == EnumTbImportType.CoverImport.getCode()) {
                ClearTableData(period, _itemTableName, EnumValidationType.InputVoiceItem.getCode());
            }

            //将进项发票明细数据转换为datatable
            //调用sqlBulk进行快速导入
            inputInvoiceItemDtoList.forEach(m -> inputVATInvoiceItemMapper.insert(m));
//            ValidateInputInvoiceItemInfo(period, userId);//todo:[validation] to be continue (neo)
        }
        return new OperationResultDto(true);
    }

    public OperationResultDto ImportInputInvoiceData(List<InputVATInvoiceDto> inputInvoiceDtoList, int importType, String userId) {
        if (inputInvoiceDtoList.size() > 0) {
            int period = inputInvoiceDtoList.get(0).getPeriodId();

            inputInvoiceDtoList.forEach(item -> {
                item.setId(UUID.randomUUID().toString());
                if (item.getCertificationDate().equals(new Date(0))) {
                    item.setCertificationDate(null);
                }

                item.setCreatorId(userId);
                item.setCreateTime(new Date());
            });

            //如果是覆盖导入，则先删除该期间的数据，再进行数据导入
            if (importType == EnumTbImportType.CoverImport.getCode()) {
                ClearTableData(period, _tableName, EnumValidationType.InputVoice.getCode());
            }

//            DealWithTheDuplicateData(inputInvoiceDtoList);//todo:[validation] to be continue (neo)

            //导入之前转换为与数据库字段一致的dto
            //将进项发票数据转换为datatable
            inputInvoiceDtoList.forEach(m -> inputVATInvoiceMapper.insert(m.extractInvoice()));
            //验证发票信息，包括重复验证和错误验证
//            ValidateInputInvoiceInfo(period, userId);//todo:[validation] to be continue (neo)
        }
        return new OperationResultDto(true);
    }

    private void ClearTableData(int period, String tableName, int importTypeId) {
        String clearCommand = "";
        if (tableName != _validationTableName) {
            //删除主数据表中的数据
            clearCommand += "DELETE FROM " + tableName + " WHERE PeriodId = " + period + ";";

            //如果是删除主表信息，则还需要删除明细表数据
            if (tableName == _tableName) {
                clearCommand += "DELETE FROM " + _itemTableName + " WHERE PeriodId = " + period + ";";
            }
        }

        //删除验证信息表中的数据
        clearCommand += "DELETE FROM " + _validationTableName + " WHERE PeriodId = " + period + " AND ImportTypeId = " + importTypeId + ";";
        exeCommand(clearCommand);
    }

    private void exeCommand(String command) {
        try (SqlSession sqlSession = dynamicSqlSessionTemplate.getSqlSessionFactory().openSession();) {
            ScriptRunner scriptRunner = new ScriptRunner(sqlSession.getConnection());
            scriptRunner.setAutoCommit(true);
            scriptRunner.runScript(new StringReader(command));
        }
    }
}
