package pwc.taxtech.atms.vat.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import javafx.geometry.Orientation;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionTemplate;
import org.reflections.util.Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dpo.PagingResultDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.ImportOutputInvoiceDto;
import pwc.taxtech.atms.dto.vatdto.QueryEvidenceDto;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.invoice.OutputInvoiceDetailMapper;
import pwc.taxtech.atms.invoice.OutputInvoiceMapper;
import pwc.taxtech.atms.service.impl.Message;
import pwc.taxtech.atms.vat.dao.ImportFileMapper;
import pwc.taxtech.atms.vat.dao.OutputVatInvoiceItemMapper;
import pwc.taxtech.atms.vat.dao.OutputVatInvoiceMapper;
import pwc.taxtech.atms.vat.dpo.OutputVATInvoiceInfoDto;
import pwc.taxtech.atms.vat.dpo.QueryOutputDto;
import pwc.taxtech.atms.vat.entity.*;

import java.io.IOException;
import java.io.StringReader;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static pwc.taxtech.atms.constant.Constant.OUTPUT_KPZT_YES;

@Service
public class OutputInvoiceServiceImpl {
    @Autowired
    private ImportFileMapper importFileMapper;
    private static final String _tableName = "OutputVATInvoice";
    private static final String _itemTableName = "OutputVATInvoiceItem";
    private static final Calendar CAL = Calendar.getInstance();
    @Autowired
    private SqlSessionTemplate dynamicSqlSessionTemplate;
    @Autowired
    private OutputVatInvoiceMapper outputVATInvoiceMapper;
    @Autowired
    private OutputVatInvoiceItemMapper outputVATInvoiceItemMapper;
    @Autowired
    private OutputInvoiceMapper outputInvoiceMapper;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private OrganizationMapper organizationMapper;
    @Autowired
    private OutputInvoiceDetailMapper outputInvoiceDetailMapper;

    public PageInfo<OutputInvoice> queryOutputInvoiceList(QueryOutputDto queryDto, String projectId) {
        Project project = projectMapper.selectByPrimaryKey(projectId);
        Organization organization = organizationMapper.selectByPrimaryKey(project.getOrganizationId());

        OutputInvoiceExample outputInvoiceExample = new OutputInvoiceExample();
        if(organization.getTaxPayerNumber() == null){
            outputInvoiceExample.createCriteria().andXFSHIsNull().andKPZTEqualTo(OUTPUT_KPZT_YES).
                    andKPRQBetween(DateUtils.getPeriodBegin(project.getYear(), queryDto.getPeriodStart()),
                            DateUtils.getPeriodEnd(project.getYear(), queryDto.getPeriodEnd()));
        } else {
            outputInvoiceExample.createCriteria().andXFSHEqualTo(organization.getTaxPayerNumber()).andKPZTEqualTo(OUTPUT_KPZT_YES).
                    andKPRQBetween(DateUtils.getPeriodBegin(project.getYear(), queryDto.getPeriodStart()),
                            DateUtils.getPeriodEnd(project.getYear(), queryDto.getPeriodEnd()));
        }


        PageHelper.startPage(queryDto.getPageInfo().getPageIndex(), queryDto.getPageInfo().getPageSize());
        List<OutputInvoice> invoices = outputInvoiceMapper.selectByExample(outputInvoiceExample);
        DecimalFormat df = new DecimalFormat("#,###.00");
        invoices.stream().forEach(x -> {
                    x.setHJJE(df.format(new BigDecimal(x.getHJJE())));
                    x.setHJSE(df.format(new BigDecimal(x.getHJSE())));
                }
        );
        PageInfo<OutputInvoice> pageInfo = new PageInfo<>(invoices);
        return pageInfo;
    }

    public OperationResultDto<List<OutputVATInvoiceInfoDto>> getExportOutputInvoiceList(QueryOutputDto queryDto) {
        OperationResultDto<List<OutputVATInvoiceInfoDto>> result = new OperationResultDto<>();
        List<OutputVATInvoiceInfoDto> finalList = getQueryList(queryDto);
        finalList = getOutputDetailInfoList(finalList);
        result.setData(finalList);
        result.setResult(true);
        return result;
    }

    public List<ImportFile> getEvidenceList(QueryEvidenceDto queryDto) {
        ImportFileExample example = new ImportFileExample();

        if (queryDto.IsCurrentPeriod) {
            if (Utils.isEmpty(queryDto.QueryString)) {
                example.createCriteria().andFileImportTypeEqualTo(2).andPeriodIdEqualTo(queryDto.PeriodId.intValue());
            } else {
                example.createCriteria().andFileImportTypeEqualTo(2).andPeriodIdEqualTo(queryDto.PeriodId.intValue()).andFileNameLike(queryDto.QueryString);
            }
        } else {
            if (Utils.isEmpty(queryDto.QueryString)) {
                example.createCriteria().andFileImportTypeEqualTo(2);
            } else {
                example.createCriteria().andFileImportTypeEqualTo(2).andFileNameLike(queryDto.QueryString);
            }
        }
        return importFileMapper.selectByExample(example);
    }

    public OperationResultDto<List<OutputVATInvoiceInfoDto>> getOutputInvoiceList(Integer period) {
        List<OutputVATInvoiceInfoDto> result = new ArrayList<>();
        OperationResultDto<List<OutputVATInvoiceInfoDto>> dtoResult = new OperationResultDto();

        result.addAll(outputVATInvoiceMapper.queryOutputDetailWithItem(period));
        if (!result.isEmpty()) {
            result.sort(Comparator.comparing(OutputVATInvoiceInfoDto::getInvoiceId));
            OutputVatInvoiceExample example = new OutputVatInvoiceExample();
            example.createCriteria().andPeriodIdEqualTo(period.intValue());
            dtoResult.setReturnCode(Long.valueOf(outputVATInvoiceMapper.countByExample(example)).intValue());
        } else {
            dtoResult.setReturnCode(0);
        }

        dtoResult.setData(result);
        dtoResult.setResult(true);
        return dtoResult;
    }

    private List<OutputVATInvoiceInfoDto> getOutputDetailInfoList(List<OutputVATInvoiceInfoDto> query) {
        List<OutputVATInvoiceInfoDto> list = new ArrayList<>();
        List<OutputInvoiceDetail> outItem = outputInvoiceDetailMapper.selectByExample(new OutputInvoiceDetailExample());
        if (query != null && query.size() > 0) {
            for (OutputVATInvoiceInfoDto dto : query) {
                List<OutputInvoiceDetail> itemList = outItem.stream()
                        .filter(a -> a.getFPQQLSH().equals(dto.getFpqqlsh()))
//                        .sorted(Comparator.comparing(OutputVatInvoiceItem::getSeqNo))
                        .collect(Collectors.toList());

                if (itemList != null && itemList.size() > 0) {
                    OutputVATInvoiceInfoDto newDto = new OutputVATInvoiceInfoDto();
                    CommonUtils.copyProperties(dto, newDto);
//                    newDto.setCodeVersion(itemList.get(0).getCodeVersion());
                    newDto.setProductName(itemList.get(0).getSPMC());
//                    newDto.setDocumentNum(itemList.get(0).getDocumentNum());
//                    newDto.setProductStandard(itemList.get(0).getProductStandard());
                    newDto.setUnit(itemList.get(0).getDW());
                    newDto.setQuantity(itemList.get(0).getSL() == null ? null : new Double(itemList.get(0).getSL()).intValue());
                    newDto.setUnitPrice(itemList.get(0).getDJ() == null ? null : new Double(itemList.get(0).getDJ()));
                    newDto.setAmount(new BigDecimal(itemList.get(0).getJE()));
                    newDto.setTaxRate(new BigDecimal(itemList.get(0).getSLV()));
                    newDto.setTaxAmount(new BigDecimal(itemList.get(0).getSE()));
//                    newDto.setTaxClassCode(itemList.get(0).getTaxClassCode());
                    list.add(newDto);
                    if (itemList.size() > 1) {
                        for (int i = 1; i < itemList.size(); i++) {
                            OutputVATInvoiceInfoDto invoiceDto = new OutputVATInvoiceInfoDto();
                            invoiceDto.setPeriodId(dto.getPeriodId());
                            invoiceDto.setInvoiceType(null);
                            invoiceDto.setClassCode(StringUtils.EMPTY);
                            invoiceDto.setInvoiceNumber(StringUtils.EMPTY);
                            invoiceDto.setBuyerName(StringUtils.EMPTY);
                            invoiceDto.setBuyerTaxNumber(StringUtils.EMPTY);
                            invoiceDto.setBankAccount(StringUtils.EMPTY);
                            invoiceDto.setPhoneNum(StringUtils.EMPTY);
                            invoiceDto.setInvoiceDate(null);
//                            invoiceDto.setCodeVersion(itemList.get(i).getCodeVersion());
                            invoiceDto.setProductName(itemList.get(i).getSPMC());
//                            invoiceDto.setDocumentNum(itemList.get(i).getDocumentNum());
//                            invoiceDto.setProductStandard(itemList.get(i).getProductStandard());
                            invoiceDto.setUnit(itemList.get(i).getDW());
                            invoiceDto.setQuantity(itemList.get(i).getSL() == null ? null : new Double(itemList.get(i).getSL()).intValue());
                            invoiceDto.setUnitPrice(itemList.get(i).getDJ() == null ? null : new Double(itemList.get(i).getDJ()));
                            invoiceDto.setAmount(new BigDecimal(itemList.get(i).getJE()));
                            invoiceDto.setTaxRate(new BigDecimal(itemList.get(i).getSLV()));
                            invoiceDto.setTaxAmount(new BigDecimal(itemList.get(i).getSE()));
//                            invoiceDto.setTaxClassCode(itemList.get(i).getTaxClassCode());
                            list.add(invoiceDto);
                        }
                    }
                }
            }
        }
        return list;
    }

//    private List<OutputVATInvoiceInfoDto> getQueryList2(QueryOutputDto queryDto) {
//        List<OutputVATInvoiceInfoDto> rList = outputInvoiceMapper.selectOutputVATInvoiceInfoLeftJoinItem(queryDto);
//        Function<OutputVATInvoiceInfoDto, List<Object>> compositeKey = outputVATInvoiceInfoDto ->
//                Arrays.asList(outputVATInvoiceInfoDto.getInvoiceId()
//                        , outputVATInvoiceInfoDto.getClassCode()
//                        , outputVATInvoiceInfoDto.getInvoiceNumber());
//        List<OutputVATInvoiceInfoDto> groupedList = new ArrayList<>();
//        rList.stream().collect(Collectors.groupingBy(compositeKey, Collectors.toList())).forEach((k, v) -> {
//            if (v.stream().findFirst().isPresent()) {
//                OutputVATInvoiceInfoDto fItem = v.stream().findFirst().get();
//                OutputVATInvoiceInfoDto outputVATInvoiceInfoDto = new OutputVATInvoiceInfoDto();
//                CommonUtils.copyProperties(fItem, outputVATInvoiceInfoDto);
//                outputVATInvoiceInfoDto.setAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getAmount().doubleValue()).sum()));
//                outputVATInvoiceInfoDto.setTaxAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum()));
//                groupedList.add(outputVATInvoiceInfoDto);
//            }
//        });
//
//        Stream<OutputVATInvoiceInfoDto> stream = groupedList.stream();
//        if (queryDto.getInvoiceType() != null && queryDto.getInvoiceType() != 999) {
//            stream = stream.filter(a -> a.getInvoiceType().equals(queryDto.getInvoiceType()));
//        }
//        if (queryDto.getStartInvoiceDate() != null) {
//            stream = stream.filter(a -> a.getInvoiceDate().after(queryDto.getStartInvoiceDate()));
//        }
//        if (queryDto.getEndInvoiceDate() != null) {
//            stream = stream.filter(a -> a.getInvoiceDate().before(queryDto.getEndInvoiceDate()));
//        }
//        if (StringUtils.isNotBlank(queryDto.getClassCode())) {
//            stream = stream.filter(a -> a.getClassCode().contains(queryDto.getClassCode()));
//        }
//        if (StringUtils.isNotBlank(queryDto.getInvoiceNumber())) {
//            stream = stream.filter(a -> a.getInvoiceNumber().contains(queryDto.getInvoiceNumber()));
//        }
//        if (StringUtils.isNotBlank(queryDto.getBuyerName())) {
//            stream = stream.filter(a -> a.getBuyerName().contains(queryDto.getBuyerName()));
//        }
//        if (queryDto.getAmountStart() != null) {
//            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountStart()) > 0);
//        }
//        if (queryDto.getAmountEnd() != null) {
//            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountEnd()) < 0);
//        }
//        if (queryDto.getTaxAmountStart() != null) {
//            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountStart()) > 0);
//        }
//        if (queryDto.getTaxAmountEnd() != null) {
//            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountEnd()) < 0);
//        }
//        return stream.sorted(Comparator.comparing(OutputVATInvoiceInfoDto::getInvoiceDate)).collect(Collectors.toList());
//    }

    private List<OutputVATInvoiceInfoDto> getQueryList(QueryOutputDto queryDto) {
        List<OutputVATInvoiceInfoDto> rList = outputInvoiceMapper.selectOutputInvoiceInfoLeftJoinItem(queryDto);
        rList.stream().forEach(x -> {
                    CAL.setTime(x.getInvoiceDate());
                    x.setPeriodId(CAL.get(Calendar.MONTH) + 1);
                });
        Function<OutputVATInvoiceInfoDto, List<Object>> compositeKey = outputVATInvoiceInfoDto ->
                Arrays.asList(outputVATInvoiceInfoDto.getInvoiceId()
                        , outputVATInvoiceInfoDto.getClassCode()
                        , outputVATInvoiceInfoDto.getInvoiceNumber());
        List<OutputVATInvoiceInfoDto> groupedList = new ArrayList<>();
        rList.stream().collect(Collectors.groupingBy(compositeKey, Collectors.toList())).forEach((k, v) -> {
            if (v.stream().findFirst().isPresent()) {
                OutputVATInvoiceInfoDto fItem = v.stream().findFirst().get();
                OutputVATInvoiceInfoDto outputVATInvoiceInfoDto = new OutputVATInvoiceInfoDto();
                CommonUtils.copyProperties(fItem, outputVATInvoiceInfoDto);
                outputVATInvoiceInfoDto.setAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getAmount().doubleValue()).sum()));
                outputVATInvoiceInfoDto.setTaxAmount(new BigDecimal(v.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum()));
                groupedList.add(outputVATInvoiceInfoDto);
            }
        });

        Stream<OutputVATInvoiceInfoDto> stream = groupedList.stream();
        if (queryDto.getInvoiceType() != null && queryDto.getInvoiceType() != 999) {
            stream = stream.filter(a -> a.getInvoiceType().equals(queryDto.getInvoiceType()));
        }
        if (queryDto.getStartInvoiceDate() != null) {
            stream = stream.filter(a -> a.getInvoiceDate().after(queryDto.getStartInvoiceDate()));
        }
        if (queryDto.getEndInvoiceDate() != null) {
            stream = stream.filter(a -> a.getInvoiceDate().before(queryDto.getEndInvoiceDate()));
        }
        if (StringUtils.isNotBlank(queryDto.getClassCode())) {
            stream = stream.filter(a -> a.getClassCode().contains(queryDto.getClassCode()));
        }
        if (StringUtils.isNotBlank(queryDto.getInvoiceNumber())) {
            stream = stream.filter(a -> a.getInvoiceNumber().contains(queryDto.getInvoiceNumber()));
        }
        if (StringUtils.isNotBlank(queryDto.getBuyerName())) {
            stream = stream.filter(a -> a.getBuyerName().contains(queryDto.getBuyerName()));
        }
        if (queryDto.getAmountStart() != null) {
            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountStart()) > 0);
        }
        if (queryDto.getAmountEnd() != null) {
            stream = stream.filter(a -> a.getAmount().compareTo(queryDto.getAmountEnd()) < 0);
        }
        if (queryDto.getTaxAmountStart() != null) {
            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountStart()) > 0);
        }
        if (queryDto.getTaxAmountEnd() != null) {
            stream = stream.filter(a -> a.getTaxAmount().compareTo(queryDto.getTaxAmountEnd()) < 0);
        }
        return stream.sorted(Comparator.comparing(OutputVATInvoiceInfoDto::getInvoiceDate)).collect(Collectors.toList());
    }

    public OperationResultDto ImportOutputInvoiceData(ImportOutputInvoiceDto importDto, String userId) {
        if (importDto.OutputInvoiceList.size() > 0) {
            int period = importDto.OutputInvoiceList.get(0).getPeriodId();
            importDto.OutputInvoiceList.forEach(item -> {
                item.setCreatorId(userId);
                item.setCreateTime(new Date());
            });
            importDto.OutputInvoiceItemList.forEach(item -> {
                item.setCreatorId(userId);
                item.setCreateTime(new Date());
            });
            //如果是覆盖导入，则先删除该期间的数据，再进行数据导入
            if (importDto.ImportType == EnumTbImportType.CoverImport.getCode()) {
                ClearTableData(period, _tableName);
                ClearTableData(period, _itemTableName);
            }
//            DeleteOutputValidation(period, 2);//todo:[validation] to be continue (neo)
            //将发票数据转换为datatable
            importDto.OutputInvoiceList.forEach(item -> outputVATInvoiceMapper.insert(item));
            importDto.OutputInvoiceItemList.forEach(item -> outputVATInvoiceItemMapper.insert(item));

//            ValidataionDuplicationData(period, userId);//todo:[validation] to be continue (neo)
//            TaxRateValidation(period, userId);//todo:[validation] to be continue (neo)
//            StartToSendMessage(period);//todo:[validation] to be continue (neo)

        }
        return new OperationResultDto(true);

    }

    public Boolean addImportFile(ImportFile fileDto) {
        importFileMapper.insert(fileDto);
        return true;
    }

    private void ClearTableData(int period, String tableName) {
        String clearCommand = "";
        if (period > 0) {
            clearCommand = "DELETE FROM " + tableName + " WHERE PeriodId = " + period;
        } else {
            clearCommand = "TRUNCATE TABLE " + tableName;
        }
    }

    private void exeCommand(String command) {
        try (SqlSession sqlSession = dynamicSqlSessionTemplate.getSqlSessionFactory().openSession();) {
            ScriptRunner scriptRunner = new ScriptRunner(sqlSession.getConnection());
            scriptRunner.setAutoCommit(true);
            scriptRunner.runScript(new StringReader(command));
        }
    }

    public List<OutputInvoiceDetail> QueryOutputInvoiceItemList(String invoiceId) {
        OutputInvoiceExample example = new OutputInvoiceExample();
        example.createCriteria().andIDEqualTo(invoiceId);
        OutputInvoice outputInvoice = outputInvoiceMapper.selectByExample(example).get(0);

        OutputInvoiceDetailExample detailExample = new OutputInvoiceDetailExample();
        detailExample.createCriteria().andFPQQLSHEqualTo(outputInvoice.getFPQQLSH());
        List<OutputInvoiceDetail> result = outputInvoiceDetailMapper.selectByExample(detailExample);
        return result;
    }
}
