package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.vat.entity.PeriodReport;

import java.util.Calendar;
import java.util.Date;

public class ProjectContext extends FunctionBase implements FreeRefFunction {
    public ProjectContext(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 2) {
            return StringEval.EMPTY_INSTANCE;
        }

        String code = this.getStringParam(args[0], ec);
        Integer parameterInt = this.getIntParam(args[1], ec);
        Calendar calendar = Calendar.getInstance();
        Calendar calendar2 = Calendar.getInstance();
        switch (code) {
            case "TaxPayer":
                String taxPayerName =
                        SpringContextUtil.organizationMapper.selectByPrimaryKey(formulaContext.getOrganizationId()).getName();
                if (StringUtils.isNotBlank(taxPayerName)) {
                    return new StringEval(String.format(Constant.ReportBuildInStringFormat.TaxPayer, taxPayerName));
                } else {
                    return new StringEval(String.format(Constant.ReportBuildInStringFormat.TaxPayer, "      "));
                }
            case "DataTime":
                return new StringEval(String.format(Constant.DateTimeFormate, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH)));
            case "FilterDate":
                return new StringEval(String.format(Constant.DateTimeFormate, formulaContext.getYear(), formulaContext.getPeriod()));
            case "TimeInterval":
                Date beginOfPeriod = new Date(formulaContext.getYear(), formulaContext.getIsYear() ? 1 : formulaContext.getPeriod(), 1);
                calendar.setTime(beginOfPeriod);
                calendar.add(Calendar.YEAR, 1);
                calendar.add(Calendar.DAY_OF_MONTH, -1);
                calendar2.setTime(beginOfPeriod);
                calendar2.add(Calendar.MONTH, 1);
                calendar2.add(Calendar.DAY_OF_MONTH, -1);
                Date endOfPeriod = formulaContext.getIsYear() ? calendar.getTime() : calendar2.getTime();

                calendar.setTime(beginOfPeriod);
                calendar2.setTime(endOfPeriod);
                return new StringEval(String.format(Constant.ReportBuildInStringFormat.TimeInterval, formulaContext.getYear()
                        , formulaContext.getPeriod(), calendar.get(Calendar.DAY_OF_MONTH), formulaContext.getYear()
                        , formulaContext.getPeriod(), calendar2.get(Calendar.DAY_OF_MONTH)));
            case "FillForm":
                PeriodReport report = SpringContextUtil.reportMapper.selectByPrimaryKey(formulaContext.getReportId());
                if (report != null) {
                    return new StringEval(String.format(Constant.ReportBuildInStringFormat.FillForm, report.getUpdateTime().getYear()
                            , report.getUpdateTime().getMonth(), report.getUpdateTime().getDay()));
                } else {
                    return new StringEval(String.format(Constant.ReportBuildInStringFormat.FillFormStr, " ", "  ", "  "));
                }
            case "TaxPayerNo":
                String taxPayerNo = SpringContextUtil.organizationMapper.selectByPrimaryKey(formulaContext.getOrganizationId()).getTaxPayerNumber();
                if (StringUtils.isNotBlank(taxPayerNo) && taxPayerNo.length() - 1 >= parameterInt) {
                    return new StringEval(String.valueOf(taxPayerNo.charAt(parameterInt)));
                }
                break;
            default:
                break;
        }
        return StringEval.EMPTY_INSTANCE;
    }
}
