package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dto.vatdto.OutputInvoiceDataSourceDto;
import pwc.taxtech.atms.vat.dpo.OutputInvoiceDto;
import pwc.taxtech.atms.vat.dpo.OutputVATInvoiceDto;
import pwc.taxtech.atms.vat.entity.OutputInvoice;
import pwc.taxtech.atms.vat.entity.OutputInvoiceDetailExample;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/// <summary>
/// 名称：销项发票
/// 功能：从发票取数，一般取发票金额或税额汇总数
/// </summary>
/// <param name="taxRate">税率取值为17%，13%，11%，6%，5%, 3%，1.5%，若所有税率都取则为99</param>
/// <param name="invoiceType">发票类型——两种取值结果“1”专用发票和“0”除专用发票以外其他发票，若所有发票都取则为99</param>
/// <param name="amountType">取值——两种取值结果“金额”0和“税额”1</param>
/// <param name="period">会计期间（开票日期）——三种取值结果 “-1”指上期，“0”当期，“99”当前累计（从期初到目前）</param>
/// <returns></returns>
public class XXFP extends FunctionBase implements FreeRefFunction {
    public XXFP(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return null;
        }

        String taxRate = getStringParam(args[0], ec);
        int invoiceTypeParam = getIntParam(args[1], ec);
        int amountType = getIntParam(args[2], ec);
        int period = getIntParam(args[3], ec);
        String invoiceType = "004";

        String formulaExpression = "XXFP(\"" + taxRate + "\"," + invoiceTypeParam + "," + amountType + "," + period + ")";
        logger.debug(formulaExpression);

        Integer curYear = formulaContext.getYear();
        Integer yearOffset = 0;
        period = getPeriod(period, yearOffset);
        curYear = curYear + yearOffset;

        BigDecimal taxRateVal = new BigDecimal("-1");
        if (!taxRate.endsWith("%")) {
            if (taxRate.equals("99")) {
                taxRateVal = new BigDecimal("99");
            } else {
                return NumberEval.ZERO;
            }
        } else {
            taxRate = StringUtils.removeEnd(taxRate, "%");
            try {
                taxRateVal = BigDecimal.valueOf(Double.parseDouble(taxRate));
            } catch (NumberFormatException ex) {
                return NumberEval.ZERO;
            }
            taxRateVal = taxRateVal.divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP); //保留两位小数，然后四舍五入
        }

//        String dbName = SpringContextUtil.projectDao.getDbNameWithYearAndOrgId(formulaContext.getOrganizationId(), curYear);
//        List<OutputVATInvoiceDto> outputInvoice = SpringContextUtil.outputVatInvoiceMapper.getVatInvoiceWithItems(dbName);
        List<OutputInvoiceDto> outputInvoice = SpringContextUtil.outputInvoiceMapper.getOutputInvoiceWithDetail();

        if (invoiceTypeParam == 1) {
            outputInvoice = outputInvoice.stream()
//                    .filter(a -> a.getInvoice().getInvoiceType() == invoiceType)
                    .filter(a -> StringUtils.equals(a.getInvoice().getFPLXDM(), invoiceType))
                    .collect(Collectors.toList());
        } else if(invoiceTypeParam == 0){
            outputInvoice = outputInvoice.stream()
//                    .filter(a -> a.getInvoice().getInvoiceType() == invoiceType)
                    .filter(a -> !StringUtils.equals(a.getInvoice().getFPLXDM(), invoiceType))
                    .collect(Collectors.toList());
        }

        if (taxRateVal.compareTo(new BigDecimal("99")) != 0) {
            BigDecimal finalTaxRateVal = taxRateVal;
            outputInvoice = outputInvoice.stream()
                    .filter(a -> new BigDecimal(a.getInvoiceDeatil().getSLV()).compareTo(finalTaxRateVal) == 0)
                    .collect(Collectors.toList());
        }

        if (period != 99) {
            int finalPeriod = period;
            outputInvoice = outputInvoice.stream().filter(a -> getPeriodFromDate(a.getInvoice().getKPRQ()) == finalPeriod).collect(Collectors.toList());
        }

        List<OutputInvoiceDataSourceDto> dataSource = new ArrayList<>();
        for (OutputInvoiceDto x : outputInvoice) {
            OutputInvoiceDataSourceDto outputInvoiceDataSourceDto = new OutputInvoiceDataSourceDto();
            outputInvoiceDataSourceDto.setAmount(FormulaHelper.roundValue(new BigDecimal(x.getInvoiceDeatil().getJE()),
                    KeyValueConfigResultType.Accounting, null, formulaContext));
            outputInvoiceDataSourceDto.setResultType(KeyValueConfigResultType.Accounting.getCode());
            outputInvoiceDataSourceDto.setTaxAmount(FormulaHelper.roundValue(new BigDecimal(x.getInvoiceDeatil().getSE()),
                    KeyValueConfigResultType.Accounting, null, formulaContext));
            outputInvoiceDataSourceDto.setTaxRate(new BigDecimal(x.getInvoiceDeatil().getSLV()));
            outputInvoiceDataSourceDto.setBuyerName(x.getInvoice().getGFMC());
            outputInvoiceDataSourceDto.setInvoiceCode(x.getInvoice().getFPDM());
            outputInvoiceDataSourceDto.setInvoiceNumber(x.getInvoice().getFPHM());
            outputInvoiceDataSourceDto.setInvoiceDate(DateUtils.strToDate(x.getInvoice().getKPRQ()));
            outputInvoiceDataSourceDto.setInvoiceType(invoiceTypeParam);
            outputInvoiceDataSourceDto.setPeriod(getPeriodFromDate(x.getInvoice().getKPRQ()));
            outputInvoiceDataSourceDto.setName(Constant.DataSourceName.OutputInvoiceDataSource);
            outputInvoiceDataSourceDto.setOperationType(EnumOperationType.Single.getCode());
            dataSource.add(outputInvoiceDataSourceDto);
        }

        if (amountType == 0) {
            double val = dataSource.stream().mapToDouble(a -> a.getAmount().doubleValue()).sum();
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.OutputInvoiceDataSourceDto,
                    new BigDecimal(val), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(val), dataSoureId, formulaContext.getProjectId());
            return new NumberEval(val);
        } else if (amountType == 1) {
            double val = dataSource.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum();
            Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.OutputInvoiceDataSourceDto,
                    new BigDecimal(val), period, formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal(val), dataSoureId, formulaContext.getProjectId());
            return new NumberEval(val);
        }
        return NumberEval.ZERO;
    }

    private int getPeriodFromDate(String dateStr){
        if(StringUtils.isBlank(dateStr)){
            return -1;
        }
        Date date = DateUtils.strToDate(dateStr);
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.MONTH) + 1;
    }
}
