package pwc.taxtech.atms.dao;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Result;
import org.apache.ibatis.annotations.Results;
import org.apache.ibatis.annotations.Select;

import pwc.taxtech.atms.MyMapper;
import pwc.taxtech.atms.dpo.DimensionValueOrgDto;
import pwc.taxtech.atms.dpo.UserOrgRoleDto;
import pwc.taxtech.atms.entity.UserOrganization;

@Mapper
public interface MyUserMapper extends MyMapper {

    @Select("select ur.id as id, ur.Is_Accessible as IsAccessible, ur.Has_Original_Role as HasOriginalRole, "
            + "org.id as OrganizationID, org.Name as OrganizationName from USER_ORGANIZATION ur "
            + "join \"ORGANIZATION\" org on ur.Organization_ID = org.id "
            + "where ur.User_ID = #{userId} and org.Is_Active = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "IsAccessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "HasOriginalRole", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "OrganizationID", property = "organizationID", javaType = String.class),
            @Result(column = "OrganizationName", property = "organizationName", javaType = String.class) })
    List<Map<String, Object>> selectUserOrganizationMapByUserId(@Param(value = "userId") String userId);

    @Select("select urd.id, urd.Dimension_Value_ID, urd.Dimension_ID, urd.User_ID, urd.Is_Accessible, urd.Has_Original_Role, u.User_Name "
            + "from USER_DIMENSION_VALUE urd " + "join \"USER\" u on urd.USER_ID = u.id " + "where u.Status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "Dimension_Value_ID", property = "dimensionValueID", javaType = String.class),
            @Result(column = "Dimension_ID", property = "dimensionId", javaType = String.class),
            @Result(column = "User_ID", property = "userID", javaType = String.class),
            @Result(column = "Is_Accessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "Has_Original_Role", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "User_Name", property = "userName", javaType = String.class) })
    List<Map<String, Object>> selectUserDimensionValueMap();

    @Select("select x.Role_ID, y.Name as RoleName from USER_ROLE x " + "join ROLE y on x.Role_ID = y.id "
            + "where x.User_ID = #{userId} and x.Project_ID is NULL")
    @Results({ @Result(column = "Role_ID", property = "roleID", javaType = String.class),
            @Result(column = "RoleName", property = "roleName", javaType = String.class) })
    List<Map<String, String>> selectUserRoleListByUserId(@Param(value = "userId") String userId);

    @Select("select q.id, q.Organization_ID, q.User_ID, q.Is_Accessible, q.Has_Original_Role from USER_ORGANIZATION q join \"USER\" u on q.User_ID = u.id "
            + "where u.Status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "Organization_ID", property = "organizationID"),
            @Result(column = "Has_Original_Role", property = "hasOriginalRole"),
            @Result(column = "User_ID", property = "userID"),
            @Result(column = "Is_Accessible", property = "isAccessible") })
    List<UserOrganization> selectUserOrganizationByUserId();

    @Select("select p.Dimension_Value_ID, p.Dimension_ID, d.Name as DimensionName, p.Dimension_Value_Name as Dimension_Value "
            + "from (select id as Dimension_Value_ID, Name as Dimension_Value_Name, Is_Active, #{dimensionId} as Dimension_ID, "
            + "Create_Time, Update_Time from BUSINESS_UNIT where Is_Active = 1 " + "union "
            + "select id as DimensionValueID, Name as DimensionValueName, Is_Active, Dimension_ID, Create_Time, Update_Time "
            + "from DIMENSION_VALUE) p " + "join DIMENSION d on p.Dimension_ID = d.id " + "where d.Is_Active = 1")
    @Results({ @Result(column = "DimensionValueID", property = "dimensionValueID"),
            @Result(column = "DimensionID", property = "dimensionId"),
            @Result(column = "DimensionName", property = "dimensionName"),
            @Result(column = "DimensionValue", property = "dimensionValue") })
    List<DimensionValueOrgDto> selectUserRoleDimensionValueByDimensionId(
            @Param(value = "dimensionId") String dimensionId);

    @Select("select urd.id, urd.Organization_ID as OrgID, urd.User_ID, urd.Is_Accessible, urd.Has_Original_Role, u.User_Name as UserName "
            + "from USER_ORGANIZATION urd join \"USER\"  u on urd.User_ID = u.id where u.Status = 1")
    @Results({ @Result(id = true, column = "id", property = "id"),
            @Result(column = "OrgID", property = "orgID", javaType = String.class),
            @Result(column = "User_ID", property = "userID", javaType = String.class),
            @Result(column = "Is_Accessible", property = "isAccessible", javaType = Boolean.class),
            @Result(column = "Has_Original_Role", property = "hasOriginalRole", javaType = Boolean.class),
            @Result(column = "User_Name", property = "userName", javaType = String.class) })
    List<Map<String, Object>> selectUserOrganizationMap();

    @Select("select ur.User_ID, r.id as RoleID, r.Name as RoleName from USER_ROLE ur join \"ROLE\" r on ur.Role_ID = r.id")
    @Results({ @Result(column = "User_ID", property = "userID"), @Result(column = "RoleID", property = "roleID"),
            @Result(column = "RoleName", property = "roleName") })
    List<UserOrgRoleDto> selectUserOrgRoleDto();
}
