﻿basicDataModule
.controller('WordLibraryManageController', ['$scope', '$log', 'SweetAlert', 'keywordmapService', 'uiGridConstants', 'stdAccountService', '$translate',
    function ($scope, $log, SweetAlert, keywordmapService, uiGridConstants, stdAccountService, $translate) {
        'use strict';

        var loadstdAccountList = function () {
            stdAccountService.GetStdAccountList().success(function (data) {
                data.forEach(function (row) {
                    row.optionText = row.code + ' ' + row.fullName;
                });

                $scope.stdAccountList = data;
            });
        };

        var loadKeyWordMapList = function () {

            keywordmapService.get(1).success(function (data) {
                $scope.gridOptions.data = data;
                $scope.gridOptions.noData = data.length === 0;
                $log.debug('keywordmapService.get...');
            });
        };

        var loadSingleKeyWordMap = function () {
            var selectedKeywordMapList = $scope.gridApi.selection.getSelectedRows();

            if (selectedKeywordMapList && selectedKeywordMapList.length > 0) {
                // 选中
                keywordmapService.getsingle(selectedKeywordMapList[0].id).success(function (data) {
                    $scope.editModel = data;
                    $scope.editModel
                    $scope.isAdd = false;
                    $scope.selectAccount = _.find($scope.stdAccountList, function (num) { return num.code === data.standardCode });

                    $('#addKeywordMapPop').modal('show');
                });
            }
            else {
                $log.debug('please select one keywordMap...');
            }
        };

        var changesStdAccountSelect = function () {
            if ($scope.selectAccount) {
                $scope.editModel.name = $scope.selectAccount.name;
                $scope.editModel.stdFullName = $scope.selectAccount.fullName;
                $scope.editModel.standardCode = $scope.selectAccount.code;
            } else {
                $scope.editModel.name = '';
                $scope.editModel.stdFullName = '';
                $scope.editModel.standardCode = '';
            }
        };

        var save = function () {
            if (!($('#addKeywordMapForm').valid())) {
                return;
            }

            $scope.editModel.isActive = 1;
            if ($scope.isAdd) {
                keywordmapService.add($scope.editModel).success(function (data) {
                    if (!data.result) {
                        var errorList = [];

                        if (data.data != null && data.data.length > 0) {
                            for (var i = 0; i < data.data.length; i++) {
                                if (!data.data[i].result && data.data[i].resultMsg) {
                                    var fullNameList = _.map(data.data[i].data, function (num) { return num.fullName; });
                                    var fullNameListStr = fullNameList.join(',', fullNameList);
                                    var error = $translate.instant(data.data[i].resultMsg) + '';
                                    error = error.formatObj({ code: $scope.editModel.standardCode, fullNameList: fullNameListStr });
                                    errorList.push(error);
                                }
                            }
                        }

                        SweetAlert.info("region", errorList.join(';', errorList));
                        return;
                    };

                    $log.debug(data);
                    $('#addKeywordMapPop').modal('hide');
                    loadKeyWordMapList();
                });
            } else {
                keywordmapService.update($scope.editModel).success(function (data) {
                    if (!data.result) {
                        SweetAlert.info('info', $translate.instant(data.resultMsg));
                        return;
                    }

                    $('#addKeywordMapPop').modal('hide');
                    loadKeyWordMapList();
                });
            }
        };

        var search = function () {
            $scope.gridApi.grid.refresh();
        };

        // 新增
        var newKeywordMap = function () {
            $scope.editModel = {};
            $scope.isAdd = true;
            $scope.selectAccount = null;
            resetErrorStatus();
            $('#addKeywordMapPop').modal('show');
        };

        var resetErrorStatus = function () {
            var currentForm = $('#addKeywordMapForm');
            $.each(currentForm.children(), function (index, element) {

                // 去除div
                $(element).children('div').removeClass('has-error');
            });
            validator.resetForm();
        };

        var resources = {
            stdCodeRequired: $translate.instant('AccountKeywordStandardCodeRequired'),
            fullNameRequired: $translate.instant('AccountKeywordTextRequired')
        };


        var validator = $("#addKeywordMapForm").validate({
            errorClass: "has-error",
            rules: {
                stdCode: {
                    required: true
                },
                fullName: {
                    required: true
                }
            },
            messages: {
                stdCode: {
                    required: resources.stdCodeRequired
                },
                fullName: {
                    required: resources.fullNameRequired
                }
            },
            errorPlacement: function (error, element) {
                setErrorStyle(error, element);
            }
        });

        var setErrorStyle = function (error, element) {
            if (element.hasClass('has-error')) {
                element.parent().addClass('has-error');
                error.insertAfter(element);
                error.addClass('label');
                // error.addClass('label-danger');
            }
        };

        var modifyIsActive = function () {
            var selectedKeywordMapList = $scope.gridApi.selection.getSelectedRows();

            if (selectedKeywordMapList && selectedKeywordMapList.length > 0) {

                SweetAlert.swal({
                    title: $translate.instant('ComfirmDisable') + '?',
                    text: $translate.instant('ComfirmAccountMappingKeywordDisable').formatObj({ code: selectedKeywordMapList[0].standardCode, name: selectedKeywordMapList[0].name, fullName: selectedKeywordMapList[0].fullName }),
                    type: "warning",
                    showCancelButton: true,
                    confirmButtonColor: "#DD6B55", confirmButtonText: $translate.instant('Confirm'),
                    cancelButtonText: $translate.instant('Cancel'),
                    closeOnConfirm: false,
                    closeOnCancel: true
                },
                function (isConfirm) {
                    if (isConfirm) {
                        // 选中
                        var model = selectedKeywordMapList[0];
                        model.isActive = model.isActive === 1 ? 0 : 1;
                        keywordmapService.isactive(model).success(function (data) {
                            if (!data.result) {
                                SweetAlert.info('info', $translate.instant(data.resultMsg));
                                return;
                            } else {
                                SweetAlert.swal($translate.instant('ComfirmDisable') + '!', $translate.instant("ComfirmDisableSuccess"), "success");
                                loadKeyWordMapList();
                            }
                        });
                    }
                });
            }
            else {
                $log.debug('please select one keywordMap...');
            }
        };

        (function initialize() {
            $log.debug('WordLibraryManageController.ctor()...');
            $scope.stdAccountList = [];
            $scope.search = search;
            $scope.newKeywordMap = newKeywordMap;
            $scope.changesStdAccountSelect = changesStdAccountSelect;
            $scope.save = save;
            $scope.loadSingleKeyWordMap = loadSingleKeyWordMap;
            $scope.modifyIsActive = modifyIsActive;

            loadstdAccountList();
            loadKeyWordMapList();

            $scope.gridOptions = {
                rowHeight: 45,
                selectionRowHeaderWidth: 45,
                enableFullRowSelection: true,
                enableRowSelection: true,
                enableSorting: false,
                enableFiltering: false,
                enableColumnMenus: false,
                enableRowHeaderSelection: false,
                multiSelect: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                columnDefs: [
	              {
	                  field: 'standardCode', name: $translate.instant('StandardAccountCode'), width: '20%', headerCellClass: 'header-cell-class',
	                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.standardCode}}</span></div>'
	              },
	              {
	                  field: 'name', name: $translate.instant('StandardAccountName'), width: '40%', headerCellClass: 'header-cell-class',
	                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.name}}</span></div>'
	              },
	              {
	                  field: 'fullName', name: $translate.instant('AccountKeywordText'), width: '40%', headerCellClass: 'header-cell-class',
	                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.fullName}}</span></div>'
	              }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    $scope.gridApi.grid.registerRowsProcessor($scope.singleFilter, 200);
                }
            };

            $scope.singleFilter = function (renderableRows) {
                var matcher = new RegExp($scope.searchText);
                renderableRows.forEach(function (row) {
                    var match = false;
                    ['standardCode', 'name', 'fullName'].forEach(function (field) {
                        if (row.entity[field].match(matcher)) {
                            match = true;
                        }
                    });
                    if (!match) {
                        row.visible = false;
                    }
                });
                return renderableRows;
            };

        })();

    }
]);