﻿systemConfigurationModule.factory('ruleEngineComparer', ['ruleEngineService', function (ruleEngineService) {
    var itemState = {
        Removed: 1,
        Modified: 2,
        Added: 3,
        NoChanged: 4
    };
    function compareTaxPayerReportRule(originalItems, updatedItems) {
        var taxPayerReportRuleDtoOperations = [];
        var orginalMaps = {};
        var updatedMaps = {};

        originalItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                orginalMaps[item.id] = item;
            }
        });

        updatedItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                updatedMaps[item.id] = item;
            }
        });

        for (itemKey in orginalMaps) {
            var state = itemState.NoChanged;
            if (orginalMaps.hasOwnProperty(itemKey) === true) {
                if (updatedMaps.hasOwnProperty(itemKey) === true) {

                    var originalItem = orginalMaps[itemKey];
                    var updatedItem = updatedMaps[itemKey];

                    if (originalItem.taxPayerType !== updatedItem.taxPayerType) {
                        state = itemState.Modified;
                    }

                    if (originalItem.templateGroupId !== updatedItem.templateGroupId) {
                        state = itemState.Modified;
                    }

                    if (_.difference(originalItem.orgs, updatedItem.orgs).length > 0 || _.difference(updatedItem.orgs,originalItem.orgs).length > 0) {
                        state = itemState.Modified;
                    }
                }
                else {
                    state = itemState.Removed;
                }

                if (state === itemState.Removed) {
                    taxPayerReportRuleDtoOperations.push({action:'Delete',taxPayerReportRule:originalItem});
                } else if (state === itemState.Modified) {
                    taxPayerReportRuleDtoOperations.push({action:'Update',taxPayerReportRule:updatedItem});
                }
            }
        }

        updatedItems.forEach(function(item){
            if (item.isNotPersistent === true) {
                item.isNotPersistent = false;
                taxPayerReportRuleDtoOperations.push({action:'Add',taxPayerReportRule:item})
            }
        });

        return taxPayerReportRuleDtoOperations;
    }

    function compareTaxRuleSetting(originalItems, updatedItems) {
        var taxRuleSettingsOperations = [];
        var orginalMaps = {};
        var updatedMaps = {};

        originalItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                orginalMaps[item.id] = item;
            }
        });

        updatedItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                updatedMaps[item.id] = item;
            }
        });

        for (itemKey in orginalMaps) {
            var state = itemState.NoChanged;
            if (orginalMaps.hasOwnProperty(itemKey) === true) {
                if (updatedMaps.hasOwnProperty(itemKey) === true) {

                    var originalItem = orginalMaps[itemKey];
                    var updatedItem = updatedMaps[itemKey];
                    updatedItem.isNotAdded = true;

                    if (originalItem.groupName !== updatedItem.groupName) {
                        state = itemState.Modified;
                    }

                    if (originalItem.taxBase !== updatedItem.taxBase) {
                        state = itemState.Modified;
                    }

                    if (originalItem.taxRate !== updatedItem.taxRate) {
                        state = itemState.Modified;
                    }
                    
                    if (_.difference(originalItem.orgs, updatedItem.orgs).length > 0 || _.difference(updatedItem.orgs,originalItem.orgs).length > 0) {
                        state = itemState.Modified;
                    }
                }
                else {
                    state = itemState.Removed;
                }

                if (state === itemState.Removed) {
                    taxRuleSettingsOperations.push({ action: 'Delete', taxRuleSetting: originalItem });
                } else if (state === itemState.Modified) {
                    taxRuleSettingsOperations.push({ action: 'Update', taxRuleSetting: updatedItem });
                }
            }
        }

        updatedItems.forEach(function (item) {
            if (item.isNotPersistent === true) {
                item.isNotPersistent = false;
                taxRuleSettingsOperations.push({ action: 'Add', taxRuleSetting: item })
            }
        });

        return taxRuleSettingsOperations;
    }

    return {
        compareTaxRuleSetting: compareTaxRuleSetting,
        compareTaxPayerReportRule: compareTaxPayerReportRule
    }
}]);