commonModule.
controller('editFormulaModalController', ['$scope', '$log', '$translate', '$location', '$timeout', '$interval', '$filter', '$q', 'SweetAlert',
    '$uibModal', 'serviceTypeService', 'formulaService',
    function($scope, $log, $translate, $location, $timeout, $interval, $filter, $q, SweetAlert, $uibModal, serviceTypeService, formulaService) {
        var allCategory = {
            id: "-1",
            name: $translate.instant("All"),
            prefix: "ALL",
            isActive: true
        };

        var thisConstant = {
            serviceTypeSelect: 'serviceTypeSelect',
            formulaDxList: 'formulaDxList',
            formulaNameGet: 'GET'
        };

        var thisData = {
            serviceTypeList: [],
            formulaList: [],
            formulaTreeView: null,
        };

        var thisDataService = {
            newModel: function() {
                var editModel = {};

                return editModel;
            },
            save: function() {
                // if ($scope.selectFormula) {
                //     $log.debug('选中的公式为:' + JSON.stringify($scope.selectFormula));
                // }

                thisModalService.close($scope.selectFormula);
            }
        };


        var thisModalService = {
            modalInstance: null,
            open: function(editModel) {
                $scope.editModel = editModel;
                mainModule.resetSelectedFormula();

                var container;
                if ($scope.containerSelector) {
                    container = angular.element($scope.containerSelector);
                }
                else {
                    container = angular.element('body');
                }

                if (container.length > 1) {
                    container = angular.element(container[0]);
                }

                var modalInstance = $uibModal.open({
                    animation: true,
                    backdrop: false,
                    ariaLabelledBy: 'modal-title',
                    ariaDescribedBy: 'modal-body',
                    templateUrl: 'editFormulaModal.html',
                    appendTo: container,
                    scope: $scope,
                    windowClass: 'edit-formula-modal-wrapper',
                    resolve: {
                        editModel: editModel
                    }
                });

                modalInstance.closed.then(function () {
                    if ($scope.isUpdate) {
                        $scope.onClosed({
                            formula: $scope.selectFormula
                        });
                    }
                });

                modalInstance.result.then(function(data) {
                    $scope.editModel = data;
                    $scope.operateType = null;
                    $scope.isUpdate = true;
                }, function() {
                    $scope.operateType = null;
                    $scope.isUpdate = false;
                    $log.info('Modal dismissed at: ' + new Date());
                });

                thisModalService.modalInstance = modalInstance;

                // 加载服务类型
                $timeout(function () {
                    var promise;
                    if (thisData.serviceTypeList && thisData.serviceTypeList.length > 0) {
                    
                        mainModule.renderServiceTypeSelect(thisData.serviceTypeList);
                        promise = $q.when();
                    } else {
                        promise = mainModule.loadServiceType();
                    }

                    promise = promise.then(function () {
                        thisData.formulaList = $scope.formulaList;
                        $scope.formulaTranslatorModel.formulaList = thisData.formulaList;

                        // 确定
                        $scope.save = thisDataService.save;

                        // 取消
                        $scope.cancel = thisModalService.cancel;

                        // 加载公式
                        if (!thisData.formulaList || thisData.formulaList.length === 0) {
                            return mainModule.loadFormulaList();
                        }
                        else {
                            return $q.when();
                        }
                    });

                    promise = promise.then(function () {
                        mainModule.searchFormula();

                        $(".edit-formula-modal-wrapper .modal-content").css('top', '0px');
                        $(".edit-formula-modal-wrapper .modal-content").css('left', '0px');
                        $(".edit-formula-modal-wrapper .modal-content").draggable({
                            cursor: "cursor",
                            scroll: true,
                            cancel: '.btn, input, .form-control, .formula-text, .close, div.label'
                        });

                    });
                }, 10);
            },
            newModel: function() {
                var editModel = thisDataService.newModel();
                thisModalService.open(editModel);
            },
            close: function(result) {
                if (thisModalService.modalInstance) {
                    thisModalService.modalInstance.close(result);
                }
            },
            cancel: function() {
                if (thisModalService.modalInstance) {
                    thisModalService.modalInstance.dismiss('cancel');
                }
            }
        };

        var mainModule = {
            main: function() {

                $scope.formulaTranslatorModel = {
                    formulaList: [],
                    formulaName: '',
                    formulaParams: ''
                };


                mainModule.watchFunction();
            },
            watchFunction: function() {
                $scope.$watch('operateType', function(newValue, oldValue) {
                    if (newValue) {
                        thisModalService.newModel();
                    }
                });
            },
            loadServiceType: function() {
                return serviceTypeService.getServiceTypeList().success(function(data) {

                    // 只拿有效的
                    data = _.filter(data, function(row) {

                        return row.isActive;
                    });

                    if (data) {
                        data.unshift(allCategory);
                    
                        thisData.serviceTypeList = data;
                        mainModule.renderServiceTypeSelect(data);
                    }
                });
            },

            loadFormulaList: function() {
                return formulaService.getAll().success(function(data) {

                    var tempList;
                    if (data && data.data) {
                        tempList = _.map(data.data, function (row) {
                            return {
                                id: row.id,
                                code: row.formulaName,
                                name: row.chineseName,
                                description: row.description,
                                englishName: row.englishName,
                                requiredParamNum: row.requiredParamNum,
                                params: ''
                            };
                        });
                    }
                    else {
                        tempList = [];
                    }

                    thisData.formulaList = tempList;
                    $scope.formulaTranslatorModel.formulaList = tempList;
                });
            },

            renderServiceTypeSelect: function(data) {
                var value = data && data.length > 0 ? data[0].id : null;
                $('#' + thisConstant.serviceTypeSelect).dxSelectBox({
                    dataSource: data,
                    valueExpr: 'id',
                    displayExpr: 'name',
                    value: value,
                    onSelectionChanged: function(args) {
                        $scope.editModel.searchText = '';
                        mainModule.resetSelectedFormula();
                        mainModule.searchFormula();
                    }
                });
            },

            resetServiceType: function () {
                var o = $('#' + thisConstant.serviceTypeSelect).dxSelectBox('option');
                var onSelectionChanged = o.onSelectionChanged;
                $('#' + thisConstant.serviceTypeSelect).dxSelectBox({ onSelectionChanged: null });
                $('#' + thisConstant.serviceTypeSelect).dxSelectBox({ value: allCategory.id });
                $('#' + thisConstant.serviceTypeSelect).dxSelectBox({ onSelectionChanged: onSelectionChanged });
            },

            resetSelectedFormula: function () {
                $scope.formulaTranslatorModel.params = ''
                $scope.formulaTranslatorModel.formulaName = '';
                $scope.selectFormula = null;
            },

            searchFormula: function() {

                var o = $('#' + thisConstant.serviceTypeSelect).dxSelectBox('option');
                var serviceType = o.value;

                var filterList = [];
                if ($scope.editModel.searchText) {
                    // 搜索框有值则只显示匹配输入值的公式
                    var upperText = $scope.editModel.searchText.toUpperCase();

                    filterList = _.filter(thisData.formulaList, function(row) {

                        if (row.code && row.code.toUpperCase().indexOf(upperText) > -1) {
                            return true;
                        }

                        if (row.name && row.name.toUpperCase().indexOf(upperText) > -1) {
                            return true;
                        }

                        return false;
                    });

                    filterList = mainModule.filterServiceType(allCategory.id, filterList);


                } else {
                    // 搜索框无值则显示选择的税种对应的公式
                    filterList = mainModule.filterServiceType(serviceType, thisData.formulaList);
                }

                mainModule.renderFormulaDxTreeView(filterList);
            },

            searchFormulaByName: function () {
                
                mainModule.resetServiceType();
                mainModule.searchFormula();
            },

            filterServiceType: function(serviceType, filterSearchList) {
                if (!filterSearchList || filterSearchList.length === 0) {
                    filterSearchList = thisData.formulaList;
                }

                var filterList = _.filter(filterSearchList, function(row) {
                    if (row.code === thisConstant.formulaNameGet) {
                        // get 函数不展示
                        return false;
                    }

                    if (serviceType === allCategory.id) {
                        // 选择所有类别时
                        return true;
                    }

                    if (!row.serviceType) {

                        // 不知道服务类型的，也返回
                        return true;
                    }

                    if (row.serviceType && row.serviceType === serviceType) {
                        return true;
                    }

                    return false;
                });

                return filterList;
            },

            renderFormulaDxTreeView: function(data) {

                data = angular.copy(data);
                data.forEach(function(row) {
                    row.label = row.code + '（' + row.name + ')';
                });

                var $treeView = $('#' + thisConstant.formulaDxList).dxTreeView({
                    dataSource: data,
                    keyExpr: "id",
                    parentIdExpr: "parentID",
                    displayExpr: "label",
                    selectByClick: true,
                    expandAllEnabled: true,
                    selectNodesRecursive: true,
                    showCheckBoxesMode: "none",
                    selectionMode: 'single',
                    height: '130px',
                    noDataText: '',
                    onItemSelectionChanged: function (args) {
                        $scope.$apply(function () {
                            var itemData = args.itemData;
                            $scope.formulaTranslatorModel.formulaName = itemData.code;
                            // $scope.formulaTranslatorModel.params = itemData.params;
                            $scope.selectFormula = itemData;
                        });
                    }
                });

                thisData.formulaTreeView = $treeView.dxTreeView("instance");
            },
        };

        (function initialize() {
            mainModule.main();

            $scope.searchFormulaByName = mainModule.searchFormulaByName;
        })();
    }
]);