﻿// Formula parameter selector for formula editor
commonModule.directive('formulaParamSelector', ['$log', '$q', '$timeout', '$translate', '$uibModal', 'enums', 'formulaTranslateUtil',
    'stdAccountService', 'enterpriseAccountService',
    function ($log, $q, $timeout, $translate, $uibModal, enums, formulaTranslateUtil, stdAccountService, enterpriseAccountService) {
        'use strict';
        $log.debug('formulaParamSelector.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/common/controls/formula-param-selector/formula-param-selector.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                isShow: '=',
                formulaList: '=*',
                formulaName: '=',
                formulaParams: '=',
                accountDataSource: '=?',
                containerSelector: '@?',
                selectorOptions: '=?',
                selectorApi: '=?' // In: onSelectorClose, onSelectorConfirm, getFormulaString
            },
            link: function (scope, element, attr) {
                $log.debug('formulaParamSelector.link()...');
                var options = {
                    animation: true,
                    backdrop: false,
                    ariaLabelledBy: 'modal-title',
                    ariaDescribedBy: 'modal-body',
                    templateUrl: 'formulaParamModal.html',
                    scope: scope,
                    size: 'opened-param-selector',
                    windowClass: 'formula-param-selector-wrapper'
                };

                if (scope.containerSelector) {
                    options.appendTo = angular.element(scope.containerSelector);
                }
                else {
                    options.appendTo = angular.element('body');
                }

                if (options.appendTo.length > 1) {
                    options.appendTo = angular.element(options.appendTo[0]);
                }

                var thisModalService = {
                    modalInstance: null,
                    open: function () {
                        var modalInstance = $uibModal.open(options);

                        modalInstance.closed.then(function () {
                            if (scope.isConfirm) {
                                selectorConfirm();
                            }
                            else {
                                selectorClose();
                            }
                        });

                        modalInstance.result.then(function () {
                            scope.isConfirm = true;
                            scope.isShow = false;
                            $log.info('Modal confirmed at: ' + new Date());
                        }, function () {
                            scope.isConfirm = false;
                            scope.isShow = false;
                            $log.info('Modal dismissed at: ' + new Date());
                        });

                        thisModalService.modalInstance = modalInstance;

                        $timeout(function () {
                            $(".formula-param-selector-wrapper .modal-content").draggable({
                                cursor: "cursor",
                                scroll: true,
                                cancel: '.btn, input, select, label, .form-control, .close, div.label, .formula-translator-wrapper + div'
                            });
                        }, 10);
                    },
                    close: function (result) {
                        if (thisModalService.modalInstance) {
                            thisModalService.modalInstance.close(result);
                        }
                    },
                    cancel: function () {
                        if (thisModalService.modalInstance) {
                            thisModalService.modalInstance.dismiss('cancel');
                        }
                    }
                };

                // 刷新选中的过滤凭证科目代码
                var refreshAccountCode = function () {
                    if (_.isNumber(scope.accountIndex) && !isNaN(scope.accountIndex)) {
                        if (_.isEmpty(scope.selectedAccountCodes)) {
                            scope.curFormula.params[scope.accountIndex].value = '';
                        }
                        else {
                            scope.curFormula.params[scope.accountIndex].value = scope.selectedAccountCodes[0];
                        }
                    }
                };

                // Register api functions in internalApi
                // If selectorApi passed is not undefined, we will use it as internalApi
                scope.internalApi = scope.selectorApi || {};

                scope.close = function () {
                    thisModalService.cancel();
                };

                scope.confirm = function () {
                    thisModalService.close();
                };

                scope.getParamDisplayType = function (p) {
                    if (p) {
                        if (p.paramType === enums.paramType.dropdown) {
                            return 'PleaseSelect';
                        }
                        else if (p.paramType === enums.paramType.accountCode) {
                            return 'SelectAccount';
                        }
                        else if (p.paramDataType === enums.paramType.dropdownNumber) {
                            return 'NumberType';
                        }
                        else if (p.paramDataType === enums.paramDataType.number) {
                            return 'NumberType';
                        }
                        else {
                            return 'StringType';
                        }
                    }
                };

                scope.getOptionText = function (o, displayFormat) {
                    // 格式化字符串中,{0}: option.value, {1}: option.name, {2}: option.shortName
                    var paramStr;
                    if (!_.isEmpty(displayFormat)) {
                        paramStr = displayFormat
                            .replace(/\{0\}/g, o.value)
                            .replace(/\{1\}/g, o.name)
                            .replace(/\{2\}/g, o.shortName);
                    }
                    else {
                        paramStr = o.value;
                    }

                    return paramStr;
                };

                scope.showSelect = function (paramType) {
                    if (paramType === enums.paramType.dropdown) {
                        return true;
                    }
                    else {
                        return false;
                    }
                }

                scope.showAccountSelector = function (paramType) {
                    if (paramType === enums.paramType.accountCode) {
                        return true;
                    }
                    else {
                        return false;
                    }
                };

                scope.showInputSelect = function (paramType) {
                    if (paramType === enums.paramType.dropdownNumber) {
                        return true;
                    }
                    else {
                        return false;
                    }
                };

                scope.openAccountSelect = function (paramIndex) {
                    scope.accountIndex = paramIndex;
                    scope.selectedAccountCodes.length = 0;
                    var initAccount = scope.accountDataSource
                        && _.some(scope.accountDataSource, { code: scope.curFormula.params[paramIndex].value });
                    if (initAccount) {
                        scope.selectedAccountCodes.push(scope.curFormula.params[paramIndex].value);
                    }
                    
                    scope.isAccountSelectorOpen = true;
                };

                scope.inputSelectChanged = function (param) {
                    if (param) {
                        if (param.selectValue === 'Other') {
                            param.value = null;
                            $('#param-select' + param.index).removeClass('special-input-option');
                            $('#param-input' + param.index).removeClass('hide');
                        }
                        else {
                            param.value = param.selectValue;
                            $('#param-input' + param.index).addClass('hide');
                            $('#param-select' + param.index).addClass('special-input-option');
                        }
                    }
                };

                // 设置科目代码选择器
                scope.accountSelectorOptions = {
                    height: 500,
                    top: 140,
                    width: 730
                };
                scope.accountSelectorApi = {
                    onSelectorConfirm: refreshAccountCode
                };
                scope.selectedAccountCodes = [];
                scope.isAccountSelectorOpen = false;

                var selectorClose = function () {
                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorClose)) {
                        scope.internalApi.onSelectorClose();
                    }
                };

                var selectorConfirm = function () {
                    if (scope.internalApi && _.isFunction(scope.internalApi.onSelectorConfirm)) {
                        var paramVals = _.map(scope.curFormula.params, function (p) {
                            var rtn = '';
                            if (p && p.value !== undefined && p.value !== null) {
                                if (p.paramDataType === enums.paramDataType.number) {
                                    rtn = p.value.toString();
                                }
                                else {
                                    rtn = '"' + p.value + '"';
                                }
                            }

                            return rtn;
                        });
                        scope.resultParamStr = paramVals.join(',') + ')';
                        scope.internalApi.onSelectorConfirm({
                            'formula': scope.formulaName,
                            'param': scope.resultParamStr
                        });
                    }
                };

                // Adjust the selector options
                var adjustOptions = function () {
                    // Adjust the CSS styles of the selector, currently only support width, height, top and left.
                    scope.selectorOptions = _.extend({}, scope.selectorOptions);
                    var styleStr = ".modal-opened-param-selector > .modal-content > form > .modal-body {";
                    _.chain(scope.selectorOptions).pick('width', 'height', 'top', 'left').pairs().each(function (c) {
                        styleStr = styleStr + c[0] + ':' + c[1] + 'px;';
                    });
                    scope.cssOptions = styleStr + '}';
                };

                var onFormulaChange = function () {
                    if (_.isEmpty(scope.formulaName)) {
                        scope.curFormula = {};
                        paramInfo = null;
                        return;
                    }

                    scope.curFormula = _.findWhere(scope.formulaList, { code: scope.formulaName });
                    scope.resultParamStr = scope.formulaParams;
                    var paramInfo = formulaTranslateUtil.getParamsFromStr(scope.resultParamStr);
                    if (scope.curFormula && paramInfo) {
                        scope.curFormula.params.forEach(function (p) {
                            if (p.paramType === enums.paramType.dropdownNumber) {
                                p.defaultOption = {
                                    name: $translate.instant('Other'),
                                    shortName: $translate.instant('Other'),
                                    value: 'Other'
                                };
                            }

                            if (paramInfo[p.index] && paramInfo[p.index].dataType === p.paramDataType) {
                                p.value = paramInfo[p.index].value;
                                if (!_.isString(p.value)) {
                                    p.value = p.value.toString();
                                }

                                if (p.paramType === enums.paramType.dropdownNumber) {
                                    if (!_.isEmpty(p.options) && _.some(p.options, { value: p.value })) {
                                        p.selectValue = p.value;
                                    }
                                    else {
                                        p.selectValue = 'Other';
                                    }
                                }
                            }
                            else {
                                if (p.paramType === enums.paramType.dropdownNumber) {
                                    p.selectValue = 'Other';
                                }

                                p.value = null;
                            }
                        });
                    }
                };

                var getFormulaString = function () {
                    return scope.formulaName + '(' + scope.formulaParams;
                };

                scope.$watch('isShow', function (newValue, oldValue) {
                    if (newValue) {
                        adjustOptions();

                        if (scope.internalApi) {
                            scope.internalApi.getFormulaString = getFormulaString;
                        }

                        onFormulaChange();
                        thisModalService.open();
                    }
                });

                scope.$watch('formulaName', function (newVal, oldValue) {
                    onFormulaChange();
                });

                scope.$watch('formulaParams', function (newVal, oldValue) {
                    onFormulaChange();
                });

                scope.$watchCollection('formulaList', function (newVal, oldValue) {
                    onFormulaChange();
                });
            }
        };
    }
]);