﻿commonModule.factory('dxDataGridService', ['$translate', function ($translate) {
    'use strict';

    var resize = function (instance) {
        var rootElement = instance.element();
        var gridView = instance.getView("gridView");
        var rowsView = instance.getView("rowsView");

        var rowsViewHeight = rowsView.element().height();
        var origRootHeigth = rootElement.height();
        var origGridViewHeigth = gridView.getElementHeight();
        var constHeight = origGridViewHeigth - rowsViewHeight;

        var tableElements = rowsView.getTableElements();
        if (tableElements.find("> tbody > .dx-freespace-row:visible").length > 0 && !instance.option("editing.allowUpdating")) {
            tableElements.find("> tbody > .dx-freespace-row:visible").hide();
        }

        if (instance.getController("data").items().length > 0) {
            rowsViewHeight = 0;
            _.forEach(tableElements, function (item) {
                rowsViewHeight += $(item).height();
            });
        }

        //用于测试观察控件高度变化
        //console.log("TIME: " + moment().format("HH:mm:ss.SSSS") + " TableElements height : " + rowsViewHeight + "px");

        var excpHeight = constHeight + rowsViewHeight;

        rootElement.css("maxHeight", "none");
        rootElement.css("minHeight", "none");
        var $testDiv = $("<div>").height(99999).appendTo(rootElement);
        var maxHeight = rootElement.height();
        $testDiv.remove();

        var fHeight = excpHeight > maxHeight ? maxHeight : excpHeight;
        /* 如果没有设置 overflow-y 为 hidden 则,不调整高度,防止高度过度溢出 */
        if (rootElement.css("overflow-y") === "hidden") {
            rootElement.css("maxHeight", fHeight + "px");
            rootElement.css("minHeight", fHeight + "px");
        }
        if (Math.abs(origRootHeigth - fHeight) < 2 && Math.abs(origGridViewHeigth - fHeight) < 2) {
            return;
        }

        instance.resize();
    };

    //1.在内容发生变化时,重新 resize
    var registerAutoSize = function (instance) {
        var contentReady = instance.option("onContentReady");
        var rowCollapsed = instance.option("onRowCollapsed");
        var rowExpanded = instance.option("onRowExpanded");

        instance.option("onContentReady", function (e) {
            if (contentReady)
                contentReady(e);
            resize(instance);
        });


        instance.option("onRowCollapsed", function (e) {
            if (rowCollapsed)
                rowCollapsed(e);
            resize(instance);
        });

        instance.option("onRowExpanded", function (e) {
            if (rowExpanded)
                rowExpanded(e);
            resize(instance);
        });

        $(window).on("resize", function () {
            instance.repaint();
        });
    };

    //2.列头单元格渲染完成后,如果单元格存在排序箭头则,将箭头设置靠右,文本内容居中
    var repaintHeaderCell = function (dataGridInstance) {

    };

    // 本方法仅为了解决 dxDataGrid 设置了 max-height 没有设置 height 导致的内容溢出问题
    //为 onInitialized   时添加 min-height,使 dxDataGrid 之 _hasHeight 为真,使 rowsView 高度可以通过 dxDataGrid 来计算,避免内容溢出
    //在 onRowPrepared   时取消 rowsView 的 min-height
    //在 onContentReady  时检验数据数量,当数量为 0 时,设置 rowsView min-height = 100px,以此弥补为 dxDataGrid 设置 min-height 时导致的 rowsView 高度为 0 问题
    var fixContentOverflowWithMaxHeight = function (instance) {
        var height = instance.option("height");
        //若有 height 则不再进行高度调整
        if (height && height != "auto")
            return;

        var contentReady = instance.option("onContentReady");
        var onRowPrepared = instance.option("onRowPrepared");

        instance.element().addClass("dx-datagrid-1em-min-height");

        instance.option("onContentReady", function (e) {
            if (e.component.getDataSource().items().length === 0)
                e.component.getView("rowsView").element().addClass("dx-datagrid-empty-rows-view");

            if (!!contentReady)
                contentReady(e);
        });

        instance.option("onRowPrepared", function (e) {
            var element = e.component.getView("rowsView").element();
            if (!!element)
                element.removeClass("dx-datagrid-empty-rows-view");

            if (!!onRowPrepared)
                onRowPrepared(e);
        });
    }

    //通过在 onInitialized 事件中添加 registerRowDbClick 方法注册,且 option 中应配置 onRowDbClick 回调函数
    var dxRowDbClickMock = function (instance) {
        var rowClick = instance.option("onRowClick");

        instance.option("onRowClick", function (e) {
            try {
                if (rowClick)
                    rowClick(e);
            }
            finally {
                var option = e.component.option();
                var preTimeStamp = option.rowClickTimeStamp || 0;
                option.rowClickTimeStamp = e.jQueryEvent.timeStamp;
                if (option.rowClickTimeStamp - preTimeStamp > 300 || !!option.rowDbClickIsProcessing)
                    return;

                option.rowDbClickIsProcessing = true;
                var rowDbClick = option.onRowDbClick;
                try {
                    if (rowDbClick)
                        rowDbClick(e)
                } finally {
                    option.rowDbClickIsProcessing = false;
                }
            }
        });
    };

    var BASIC_GRID_OPTIONS = {
        allowColumnResizing: true,
        editing: {
            allowAdding: false,
            allowDeleting: false,
            allowUpdating: false
        },
        focusStateEnabled: true,
        hoverStateEnabled: true,
        keyExpr: "id",
        loadPanel: {
            enabled: true
        },
        noDataText: $translate.instant('NoDataText'),
        paging: {
            enabled: false
        },
        rowAlternationEnabled: false, //单双行颜色
        scrolling: {
            mode: "infinite"
        },
        selection: {
            allowSelectAll: false,
            mode: "single"
        },
        showBorders: true,
        showColumnLines: true,
        showRowLines: true,
        sorting: {
            mode: "single"
        },
    };

    return {
        registerAutoSize: registerAutoSize,
        registerRepaintHeaderCell: repaintHeaderCell,
        resize: resize,
        fixContentOverflowWithMaxHeight: fixContentOverflowWithMaxHeight,
        registerRowDbClick: dxRowDbClickMock,
        BASIC_GRID_OPTIONS: BASIC_GRID_OPTIONS,
    };
}]);