﻿vatModule.controller('vatChartEditController', ['$log', '$scope', '$translate',
 function ($log, $scope, $timeout, $location, dashboardService, dataTransformService, chartRendererService, localStorageService, accountService) {

     'use strict';

     $log.debug('DashboardSettingController.ctor()...');

     $scope.computeMethod = ['sum', 'avg', 'count', 'max', 'min'];

     $scope.addDimension = function (name) {
         $scope.selectedDimension.push(name);

         var removeIndex = $scope.unSelectedDimension.indexOf(name);
         $scope.unSelectedDimension.splice(removeIndex, 1);

         RefreshView();
     };

     $scope.removeDimension = function (name) {
         var removeIndex = $scope.selectedDimension.indexOf(name);
         $scope.selectedDimension.splice(removeIndex, 1);

         $scope.unSelectedDimension.push(name);
         RefreshView();
     };

     $scope.dimensionAddDisabled = false;

     $scope.measuresAddDisabled = false;

     $scope.addMeasures = function (name, method) {
         var item = { field: name, method: method };

         var existItem = _.find($scope.selectedMeasures, item)

         if (existItem) {
             showErrorOperationMessage('measuresExist');
             return false;
         }

         $scope.selectedMeasures.push({ field: name, method: method });

         /*
         var removeIndex = $scope.unSelectedMeasures.indexOf(name);
         $scope.unSelectedMeasures.splice(removeIndex, 1);
         */
         RefreshView();
     };

     $scope.removeMeasures = function (field, method) {
         //$scope.unSelectedMeasures.push(field);

         for (var i = 0; i < $scope.selectedMeasures.length; i++) {
             if ($scope.selectedMeasures[i].field === field && $scope.selectedMeasures[i].method === method) {
                 $scope.selectedMeasures.splice(i, 1);
                 break;
             }
         }
         RefreshView();
     };

     $scope.changeChartType = function (chartType) {
         if (chartType === 'pie' && $scope.pieChartDisable) {
             return;
         }

         $scope.chartType = chartType;
         RefreshView();
     };

     $scope.pieChartDisable = false;

     function RefreshView() {
         $scope.initOperation = null;

         RefreshSetting();

         InitializeData(function () {
             chartRendererService.RefreshChart($scope.dataSourceList, $scope.selectedDimension, $scope.selectedMeasures, $scope.chartType,
             $scope.showValue, 'pieChartContainer', 'barChartContainer', localStorageService.get('dataValue') + '-01');
         }, failMessageCallBack);
     }

     function RefreshSetting() {
         if ($scope.chartType !== 'pie') {
             $scope.dimensionAddDisabled = false;
             $scope.measuresAddDisabled = false;

             //two dimension disable add dimension
             if ($scope.selectedDimension.length === 2) {
                 $scope.dimensionAddDisabled = true;

                 //and one measure disable add measures
                 if ($scope.selectedMeasures.length === 1) {
                     $scope.measuresAddDisabled = true;
                 }
             }

             //two measure above disable add dimension
             if ($scope.selectedMeasures.length > 1 && $scope.selectedDimension.length === 1) {
                 $scope.dimensionAddDisabled = true;
             }

             //enable or disable switch Pie Chart
             var pieChartDisable = $scope.selectedDimension.length > 1 || $scope.selectedMeasures.length > 1;
             $scope.pieChartDisable = pieChartDisable;
         }
         else {
             $scope.dimensionAddDisabled = false;
             $scope.measuresAddDisabled = false;

             if ($scope.selectedDimension.length >= 1) {
                 $scope.dimensionAddDisabled = true;
             }

             if ($scope.selectedMeasures.length >= 1) {
                 $scope.measuresAddDisabled = true;
             }
         }
     }


     var InitializeData = function (sucessCallBack, failCallBack) {
         var projectList = localStorageService.get('projectList');
         var endDate = localStorageService.get('dataValue') + '-01';
         var isMonth = $scope.selectedDimension.indexOf('month') === -1;

         dashboardService.getRangeChartData($scope.dataSourceName, projectList, endDate, isMonth)
             .success(function (dataList) {
                 if (dataList) {
                     dataTransformService.formatRegionProject(dataList, $scope.projectList);

                     $scope.dataSourceList = dataList;

                     if (sucessCallBack) {
                         sucessCallBack();
                     }
                 }
                 else {
                     failCallBack();
                 }
             })
             .error(function () {
                 failCallBack();
             });
     };


     var GetInitSetting = function (chartId, sucessCallback, failCallback) {
         dashboardService.getChartSetting(chartId).success(function (operationResult) {
             if (!operationResult.result && failCallback) {
                 //display error
                 failCallback();
                 return;
             }

             var chartSetting = operationResult.data;

             $scope.dataSourceName = chartSetting.dataSourceName;
             $scope.chartName = chartSetting.name;
             $scope.showValue = chartSetting.showValue;
             $scope.chartType = chartSetting.chartTypeValue;

             //Ignore IsArgumentField Value
             $scope.selectedDimension = dataTransformService.getDimensionList(chartSetting.argumentField);
             $scope.selectedMeasures = dataTransformService.getMeasuresValueList(chartSetting.valueField);

             $scope.unSelectedDimension = new Array();
             $scope.unSelectedMeasures = new Array();

             chartSetting.allValueFieldList.forEach(function (e) {
                 var fieldName = dataTransformService.getPrefixedFieldName(e);

                 if ($scope.selectedDimension.indexOf(fieldName) === -1) {
                     $scope.unSelectedDimension.push(fieldName);
                 }

                 //Measure can be added many times
                 $scope.unSelectedMeasures.push(fieldName);
             });

             if (sucessCallback) {
                 sucessCallback();
             }

         });

     };

     var showErrorOperationMessage = function (errorMessage) {
         $scope.operation = { result: false, message: errorMessage };

         $timeout(function () {
             $scope.operation = null;
         }, 2000);
     };


     $scope.saveSetting = function () {
         if ($scope.chartName.length === 0) {
             showErrorOperationMessage('noChartName');
             return false;
         }

         if ($scope.chartName.length >= 22) {
             showErrorOperationMessage('chartNameTooLong');
             return false;
         }

         var argumentFieldValue = dataTransformService.getDimensionValue($scope.selectedDimension);
         var valueField = dataTransformService.getMeasuresValue($scope.selectedMeasures);

         if (argumentFieldValue.length === 0 || valueField.length === 0) {
             showErrorOperationMessage('dimensionOrMeasuresError');
             return false;
         }

         dashboardService.saveSetting($scope.chartId, $scope.chartName, $scope.chartType, argumentFieldValue, valueField, $scope.showValue).success(function (operationResult) {
             showMessageAndBack(operationResult);
         });
     };

     var failMessageCallBack = function () {
         $scope.initOperation = { result: false, message: 'InitializeFail' };
     };

     var showMessageAndBack = function (operationResult) {
         $scope.operation = operationResult;

         $timeout(function () {
             $scope.operation = null;
             closeCurrentTab();
         }, 1000);
     };

     var closeCurrentTab = function () {
         $scope.$emit('event:closeCurrentTab');
     };

     // initialzation
     (function initialize() {        
         $scope.projectList = [];

         accountService.getProjectList().success(function (data) {
             $scope.projectList = data;
         });

         GetInitSetting($scope.chartId, function () {
             RefreshView();

             $('#showValue').bootstrapSwitch({ 'state': $scope.showValue, 'size': 'small' });

         }, failMessageCallBack);


         $scope.closeCurrentTab = closeCurrentTab;


         //-------------------------------------
         $scope.editSettingModel = {
             chartID:null,
             chartName:null,
             chartType: null,
             argumentFieldValue: null,
             valueField: null,
             showValue:null

         };


     }());
 }]);